package com.snowm.scanner.helpers

import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import com.snowm.scanner.models.SnowMiBeacon
import org.altbeacon.beacon.*
import java.util.HashMap
import kotlin.collections.ArrayList

class BeaconScanner(private val context: Context) : BeaconConsumer, RangeNotifier {
    private val onBeaconScannedInterfaces: HashMap<String, OnBeaconScanned> = HashMap()

    override fun unbindService(p0: ServiceConnection?) {
        return unbindService(p0)
    }

    override fun bindService(p0: Intent?, p1: ServiceConnection?, p2: Int): Boolean {
        return bindService(p0, p1, p2)
    }

    override fun onBeaconServiceConnect() {
        beaconManager.addRangeNotifier(this)
    }

    override fun didRangeBeaconsInRegion(p0: MutableCollection<Beacon>?, p1: Region?) {
        if (p0!!.isNotEmpty()) {
            val scannedBeacons = ArrayList<SnowMiBeacon>()
            p0.forEach {
                val beacon = SnowMiBeacon()
                beacon.rssi = it.rssi
                beacon.distance = it.distance
                beacon.macAddress = it.bluetoothAddress
                beacon.txPower = it.txPower
                beacon.major = it.id2.toInt()
                beacon.minor = it.id3.toInt()
                beacon.uuid = it.id1.toString().toUpperCase()
                scannedBeacons.add(beacon)
            }
            onBeaconScannedInterfaces[p1?.uniqueId]?.onScanned(scannedBeacons)
        }
    }

    override fun getApplicationContext(): Context {
        return context
    }

    private var beaconManager = BeaconManager.getInstanceForApplication(context)


    fun scanBeacons(identifier: String, onBeaconScanned: OnBeaconScanned) {
        beaconManager.beaconParsers.add(BeaconParser().setBeaconLayout("m:2-3=0215,i:4-19,i:20-21,i:22-23,p:24-24"))
        onBeaconScannedInterfaces[identifier] = onBeaconScanned
        val region = Region(identifier, null, null, null)
        beaconManager.bind(this)
        beaconManager.startRangingBeaconsInRegion(region)
    }


    fun removeScanner(identifier: String) {
        val region = Region(identifier, null, null, null)
        beaconManager.stopRangingBeaconsInRegion(region)
        onBeaconScannedInterfaces.remove(identifier)
    }

    interface OnBeaconScanned {
        fun onScanned(beacons: ArrayList<SnowMiBeacon>)
    }

}

