package com.snowm.scanner.helpers

import android.Manifest
import android.app.Activity
import android.bluetooth.BluetoothAdapter
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import com.snowm.scanner.SnowmScannerPlugin
import io.flutter.Log
import pub.devrel.easypermissions.EasyPermissions
import java.util.*

class PermissionManager internal constructor(private val context: Context) {
    private val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
    private val REQUEST_LOCATION_PERMISSION = 1
    private val perms = arrayOf(Manifest.permission.ACCESS_FINE_LOCATION)
    private val perms1 = arrayOf(Manifest.permission.BLUETOOTH)
    private val onStateRequests: HashMap<String?, SnowmScannerPlugin.OnStateRequest?> = HashMap()

    fun hasLocationPermission(): Boolean {
        return EasyPermissions.hasPermissions(context, *perms)
    }

    val bluetoothPermissionState: String
        get() = if (EasyPermissions.hasPermissions(context, *perms1)) {
            "granted"
        } else {
            "denied"
        }

    val bluetoothState: String
        get() = when {
            bluetoothAdapter == null -> {
                "unknown"
            }
            bluetoothAdapter.isEnabled -> {
                "on"
            }
            else -> {
                "off"
            }
        }

    fun bluetoothState(identifier: String?, onStateRequest: SnowmScannerPlugin.OnStateRequest?) {
        onStateRequests[identifier] = onStateRequest
        val i = Intent()
        val filter = IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED)
        context.registerReceiver(mReceiver, filter)
        mReceiver.onReceive(context, i)
    }

    fun requestLocationPermission() {
        EasyPermissions.requestPermissions((context as Activity), "Please grant the location permission", REQUEST_LOCATION_PERMISSION, *perms)
    }

    private val mReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            for (onStateRequest in onStateRequests.values) {
                try {
                    when (bluetoothAdapter!!.state) {
                        BluetoothAdapter.STATE_OFF -> onStateRequest!!.onState("off")
                        BluetoothAdapter.STATE_ON -> onStateRequest!!.onState("on")
                        else -> onStateRequest!!.onState("unknown")
                    }
                } catch (e: SecurityException) {
                    Log.d("Utils", "something is not right")
                }
            }
        }
    }

    fun removeBluetoothState(streamIdentifier: String) {
        onStateRequests.remove(streamIdentifier)
    }
}