package com.snowm.scanner.services

import android.app.*
import android.content.Context
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.os.IBinder
import android.util.Log
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import com.snowm.scanner.R
import com.snowm.scanner.helpers.BeaconScanner
import com.snowm.scanner.helpers.BeaconScanner.OnBeaconScanned
import com.snowm.scanner.helpers.MqttHelper
import com.snowm.scanner.models.SnowMiBeacon
import java.util.*
import kotlin.collections.HashMap

open class SnowMBackgroundScanningService : Service() {
    private var bundle: Bundle? = null
    private lateinit var customData: HashMap<String, Any>
    private lateinit var uuids: ArrayList<String>
    private lateinit var mqttHelper: MqttHelper
    override fun onBind(intent: Intent): IBinder? {
        return null
    }

    private var onBeaconScanned: OnBeaconScanned = object : OnBeaconScanned {
        override fun onScanned(beacons: ArrayList<SnowMiBeacon>) {
            val validBeacons: ArrayList<SnowMiBeacon> = ArrayList()
            beacons.forEach() {
                mqttHelper.sendIBeaconInformation(it)
                if (uuids.contains(it.uuid))
                    validBeacons.add(it)
            }
            if (validBeacons.count() > 0)
                onBeaconsDetected(validBeacons, customData)
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.O)
    override fun onCreate() {
        super.onCreate()
        Log.d("BackgroundService", "created")
        mqttHelper = MqttHelper(this)
        val beaconScanner = BeaconScanner(this)
        beaconScanner.scanBeacons("background", onBeaconScanned)
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        val extra = intent.extras ?: return START_NOT_STICKY
        val title = extra.getString("title")
        val body = extra.getString("body")
        uuids = extra.getStringArrayList("uuids")
        bundle = intent.extras
        customData = extra.getSerializable("customData") as HashMap<String, Any>
        createNotificationChannel(this)
        val activityIntent = Intent()
        activityIntent.setPackage(this.packageName)
        activityIntent.action = "android.intent.action.MAIN"
        activityIntent.flags = Intent.FLAG_ACTIVITY_SINGLE_TOP
        val contentIntent = PendingIntent.getActivity(this, 0, activityIntent, 0)
        val notification = NotificationCompat.Builder(this, "snowm_scanner").setContentTitle(title).setContentText(body).setSmallIcon(R.drawable.skel_logo).setContentIntent(contentIntent).build()
        startForeground(1, notification)
        return START_STICKY
    }

    private fun createNotificationChannel(context: Context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel = NotificationChannel("snowm_scanner", "Background Scanner", importance)
            channel.description = "Notification alerts for beacon scanner."
            val notificationManager = context.getSystemService(NotificationManager::class.java)
            notificationManager.createNotificationChannel(channel)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        Log.d("BackgroundService", "Destroyed")
        val beaconScanner = BeaconScanner(this)
        beaconScanner.removeScanner("background")
    }

    open fun onBeaconsDetected(beacons: ArrayList<SnowMiBeacon>, customData: HashMap<String, Any>) {}
    open fun stopScan() {
        val beaconScanner = BeaconScanner(this)
        beaconScanner.removeScanner("background")
    }

}