import Foundation
import CoreLocation

public class BeaconScanner: NSObject, CLLocationManagerDelegate{
    
    private let locationManager = CLLocationManager()
    private var listeners:Dictionary<String, (_ beacons:[SnowMBeacon])->()> = Dictionary()
    public var permissionStatus:String = "denied"
    override init(){
        super.init()
        locationManager.pausesLocationUpdatesAutomatically = false
        if #available(iOS 9.0, *) {
            locationManager.allowsBackgroundLocationUpdates = true
        }
        locationManager.delegate = self;
    }
    
    public func requestPermission(){
        locationManager.requestAlwaysAuthorization()
    }
    
    public func scanBeacons(method:String ,uuids:[String], completion: @escaping (_ beacons:[SnowMBeacon])->()){
        locationManager.requestAlwaysAuthorization()
        locationManager.startUpdatingLocation()
        self.listeners[method] = completion
        uuids.forEach { (id) in
            let uuid = UUID(uuidString: id)
            let region = CLBeaconRegion(proximityUUID: uuid!, identifier: method+"_"+id)
            locationManager.startMonitoring(for: region)
            locationManager.startRangingBeacons(in: region)
        }
    }
    
    public func removeScanner(identifier: String){
        listeners.removeValue(forKey: identifier);
        if listeners.count == 0{
            locationManager.stopUpdatingLocation()
        }
        locationManager.monitoredRegions.forEach { (region) in
            locationManager.stopMonitoring(for: region)
        }
    }
    
    
    public func locationManager(_ manager: CLLocationManager, didChangeAuthorization status: CLAuthorizationStatus) {
        switch status {
        case .restricted:
            permissionStatus = "denied"
            break
        case .authorizedAlways:
            permissionStatus = "granted"
            break
        case .authorizedWhenInUse:
            permissionStatus = "whenInUse"
            break
        case .notDetermined:
            permissionStatus = "unknown"
            break
        case .denied:
            permissionStatus = "denied"
            break
        default:
            permissionStatus = "unknown"
            break
        }
    }
    
    
    public func locationManager(_ manager: CLLocationManager, didRangeBeacons beacons: [CLBeacon], in region: CLBeaconRegion) {
        let snowmBeacons:[SnowMBeacon] = beacons.map {
            let snowmBeacon = SnowMBeacon()
            snowmBeacon.uuid = $0.proximityUUID.uuidString
            snowmBeacon.major = $0.major.intValue
            snowmBeacon.minor = $0.minor.intValue
            snowmBeacon.txPower = 0
            snowmBeacon.macAddress = "Unavailable"
            snowmBeacon.distance = $0.accuracy.distance(to: 1)
            snowmBeacon.rssi = $0.rssi
            return snowmBeacon
        }
        let method:String = region.identifier.split {$0 == "_"}.map(String.init)[0]
        if(listeners[method] != nil){
            self.listeners[method]!(snowmBeacons)
        }
    }
    
    public func locationManager(_ manager: CLLocationManager, rangingBeaconsDidFailFor region: CLBeaconRegion, withError error: Error) {
    }
}
