import Flutter
import UIKit

public class SwiftSnowmScannerPlugin: NSObject, FlutterPlugin {
    var beaconScanner = BeaconScanner()
    var utils = Utils()
    var mqttHelper = MqttHelper()
    var channel: FlutterMethodChannel?
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "snowm_scanner", binaryMessenger: registrar.messenger())
        let instance = SwiftSnowmScannerPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let method = call.method
        if method.contains("scanBeacons"){
            let arguments  = call.arguments as! NSDictionary
            let uuids:[String]  = arguments["uuids"] as! [String]
            //TODO: Trigger mqtt protocol on native side
            let enableMqtt:Bool  =  arguments["enableMqtt"] as! Bool
            beaconScanner.scanBeacons(method:method, uuids: uuids) { (beacons) -> () in
                if enableMqtt {
                    self.mqttHelper.sendBeaconData(beacons:beacons)
                }
                self.sendBeaconResponse(method:method, beacons: beacons)
            }
            result(true)
        } else if method == "requestPermission"{
            result(beaconScanner.requestPermission())
        } else if method == "permissionState"{
            result(beaconScanner.permissionStatus)
        } else if method == "bluetoothState"{
            result(utils.state)
        } else if method.contains( "bluetoothStateListener"){
            utils.bluetoothState(method: method) { (state) in
                self.sendBluetoothStateResponse(method: method, state: state)
            }
            result(true)
        } else if method == "cancelBluetoothStateListener"{
            let arguments  = call.arguments as! NSDictionary
            let streamIdentifier = arguments["methodName"] as! String
            utils.removeBluetoothState(identifier: streamIdentifier)
            result(true)
        } else if method == "cancelStream"{
            let arguments  = call.arguments as! NSDictionary
            let streamIdentifier = arguments["methodName"] as! String
            beaconScanner.removeScanner(identifier: streamIdentifier)
            result(true)
        }
    }
    
    private func sendBluetoothStateResponse(method:String, state:String){
        channel?.invokeMethod(method, arguments: state)
    }
    
    private func sendBeaconResponse(method:String, beacons: [SnowMBeacon]){
        var response: [AnyHashable:Any] = [AnyHashable:Any]();
        response["beacons"] = beacons.map{ $0.toObject()};
        channel?.invokeMethod(method, arguments: response)
    }
}
